@@ -1,31 +1,45 @@
 /*
- * Copyright (c) 1997, 2000, 01  
- *    Motoyuki Kasahara
+ * Copyright (c) 1997-2004  Motoyuki Kasahara
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the project nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "build-pre.h"
 #include "eb.h"
 #include "error.h"
+#ifdef ENABLE_EBNET
+#include "ebnet.h"
+#endif
 #include "build-post.h"
 
-#ifdef WIN32
+#if defined(DOS_FILE_PATH) && defined(HAVE_MBSTRING_H)
 /* a path may contain double-byte chars in SJIS. */
-#ifdef HAVE_MBSTRING_H /* keep Cygwin out. */
 #include <mbstring.h>
 #define strchr	_mbschr
 #define strrchr	_mbsrchr
-#endif /* HAVE_MBSTRING_H */
-#endif /* WIN32 */
+#endif
 
 
 #ifndef DOS_FILE_PATH
@@ -35,8 +49,7 @@
  * Convert a path name to an absolute path.
  */
 EB_Error_Code
-eb_canonicalize_path_name(path_name)
-    char *path_name;
+eb_canonicalize_path_name(char *path_name)
 {
     char cwd[EB_MAX_PATH_LENGTH + 1];
     char temporary_path_name[EB_MAX_PATH_LENGTH + 1];
@@ -85,8 +98,7 @@ eb_canonicalize_path_name(path_name)
  * Current version by Motoyuki Kasahara.
  */
 EB_Error_Code
-eb_canonicalize_path_name(path_name)
-    char *path_name;
+eb_canonicalize_path_name(char *path_name)
 {
     char cwd[EB_MAX_PATH_LENGTH + 1];
     char temporary_path_name[EB_MAX_PATH_LENGTH + 1];
@@ -108,7 +120,7 @@ eb_canonicalize_path_name(path_name)
 	/*
 	 * `path_name' is UNC path.  Nothing to be done.
 	 */
-    } else if (isalpha(*path_name) && *(path_name + 1) == ':') {
+    } else if (ASCII_ISALPHA(*path_name) && *(path_name + 1) == ':') {
 	/*
 	 * `path_name' has a drive letter.
 	 * Nothing to be done if it is an absolute path.
@@ -118,7 +130,7 @@ eb_canonicalize_path_name(path_name)
 	     * `path_name' is a relative path.
 	     * Covert the path name to an absolute path.
 	     */
-	    if (getdcwd(toupper(*path_name) - 'A' + 1, cwd,
+	    if (getdcwd(ASCII_TOUPPER(*path_name) - 'A' + 1, cwd,
 		EB_MAX_PATH_LENGTH + 1) == NULL) {
 		return EB_ERR_FAIL_GETCWD;
 	    }
@@ -129,7 +141,7 @@ eb_canonicalize_path_name(path_name)
 	}
     } else if (*path_name == '\\') {
 	/*
-	 * `path_name' is has no drive letter and is an absolute path.
+	 * `path_name' has no drive letter and is an absolute path.
 	 * Add a drive letter to the path name.
 	 */
 	if (getcwd(cwd, EB_MAX_PATH_LENGTH + 1) == NULL)
@@ -142,7 +154,7 @@ eb_canonicalize_path_name(path_name)
 
     } else {
 	/*
-	 * `path_name' is has no drive letter and is a relative path.
+	 * `path_name' has no drive letter and is a relative path.
 	 * Add a drive letter and convert it to an absolute path.
 	 */
 	if (getcwd(cwd, EB_MAX_PATH_LENGTH + 1) == NULL)
@@ -160,7 +172,7 @@ eb_canonicalize_path_name(path_name)
      * Unless it is "X:\", eliminate `\' in the tail of the path name.
      */
     last_backslash = strrchr(path_name, '\\');
-    if (isalpha(*path_name)) {
+    if (ASCII_ISALPHA(*path_name)) {
 	if (last_backslash != path_name + 2 && *(last_backslash + 1) == '\0')
 	    *last_backslash = '\0';
     } else {
@@ -186,33 +198,34 @@ eb_canonicalize_path_name(path_name)
  * file name using this function.
  */
 void
-eb_canonicalize_file_name(file_name)
-    char *file_name;
+eb_canonicalize_file_name(char *file_name)
 {
     char *p;
 
-    for (p = file_name; *p != '\0' && *p != '.' && *p != ';'; p++) {
-	*p = toupper(*p);
-    }
+    for (p = file_name; *p != '\0' && *p != '.' && *p != ';'; p++)
+	*p = ASCII_TOUPPER(*p);
     *p = '\0';
 }
 
 
 /*
  * Rewrite `directory_name' to a real directory name in the `path' directory.
- * 
+ *
  * If a directory matched to `directory_name' exists, then EB_SUCCESS is
  * returned, and `directory_name' is rewritten to that name.  Otherwise
  * EB_ERR_BAD_DIR_NAME is returned.
  */
 EB_Error_Code
-eb_fix_directory_name(path, directory_name)
-    const char *path;
-    char *directory_name;
+eb_fix_directory_name(const char *path, char *directory_name)
 {
     struct dirent *entry;
     DIR *dir;
 
+#ifdef ENABLE_EBNET
+    if (is_ebnet_url(path))
+	return ebnet_fix_directory_name(path, directory_name);
+#endif
+
     /*
      * Open the directory `path'.
      */
@@ -249,16 +262,14 @@ eb_fix_directory_name(path, directory_name)
 /*
  * Rewrite `sub_directory_name' to a real sub directory name in the
  * `path/directory_name' directory.
- * 
+ *
  * If a directory matched to `sub_directory_name' exists, then EB_SUCCESS
  * is returned, and `directory_name' is rewritten to that name.  Otherwise
  * EB_ERR_BAD_FILE_NAME is returned.
  */
 EB_Error_Code
-eb_fix_directory_name2(path, directory_name, sub_directory_name)
-    const char *path;
-    const char *directory_name;
-    char *sub_directory_name;
+eb_fix_directory_name2(const char *path, const char *directory_name,
+    char *sub_directory_name)
 {
     char sub_path_name[EB_MAX_PATH_LENGTH + 1];
 
@@ -274,9 +285,7 @@ eb_fix_directory_name2(path, directory_name, sub_directory_name)
  * Otherwise, add `suffix' to `path_name'.
  */
 void
-eb_fix_path_name_suffix(path_name, suffix)
-    char *path_name;
-    const char *suffix;
+eb_fix_path_name_suffix(char *path_name, const char *suffix)
 {
     char *base_name;
     char *dot;
@@ -285,7 +294,10 @@ eb_fix_path_name_suffix(path_name, suffix)
 #ifndef DOS_FILE_PATH
     base_name = strrchr(path_name, '/');
 #else
-    base_name = strrchr(path_name, '\\');
+    if (is_ebnet_url(path_name))
+	base_name = strrchr(path_name, '/');
+    else
+	base_name = strrchr(path_name, '\\');
 #endif
     if (base_name == NULL)
 	base_name = path_name;
@@ -341,8 +353,8 @@ eb_fix_path_name_suffix(path_name, suffix)
 /*
  * Rewrite `found_file_name' to a real file name in the `path_name'
  * directory.
- * 
- * If a file matched to `rarget_file_name' exists, then EB_SUCCESS
+ *
+ * If a file matched to `target_file_name' exists, then EB_SUCCESS
  * is returned, and `found_file_name' is rewritten to that name.
  * Otherwise EB_ERR_BAD_FILE_NAME is returned.
  *
@@ -350,10 +362,8 @@ eb_fix_path_name_suffix(path_name, suffix)
  * EB_MAX_DIRECTORY_NAME_LENGTH characters.
  */
 EB_Error_Code
-eb_find_file_name(path_name, target_file_name, found_file_name)
-    const char *path_name;
-    const char *target_file_name;
-    char *found_file_name;
+eb_find_file_name(const char *path_name, const char *target_file_name,
+    char *found_file_name)
 {
     char ebz_target_file_name[EB_MAX_FILE_NAME_LENGTH + 1];
     char org_target_file_name[EB_MAX_FILE_NAME_LENGTH + 1];
@@ -363,6 +373,13 @@ eb_find_file_name(path_name, target_file_name, found_file_name)
     size_t d_namlen;
     int found = FOUND_NONE;
 
+#ifdef ENABLE_EBNET
+    if (is_ebnet_url(path_name)) {
+	return ebnet_find_file_name(path_name, target_file_name,
+	    found_file_name);
+    }
+#endif
+
     strcpy(ebz_target_file_name, target_file_name);
     strcat(ebz_target_file_name, ".ebz");
     strcpy(org_target_file_name, target_file_name);
@@ -401,10 +418,10 @@ eb_find_file_name(path_name, target_file_name, found_file_name)
 	 * All the comparisons are done without case sensitivity.
 	 * We support version number ";1" only.
 	 */
-	d_namlen = NAMLEN(entry);
+	d_namlen = strlen(entry->d_name);
 	if (2 < d_namlen
 	    && *(entry->d_name + d_namlen - 2) == ';'
-	    && isdigit(*(entry->d_name + d_namlen - 1))) {
+	    && ASCII_ISDIGIT(*(entry->d_name + d_namlen - 1))) {
 	    d_namlen -= 2;
 	}
 	if (1 < d_namlen && *(entry->d_name + d_namlen - 1) == '.')
@@ -458,12 +475,8 @@ eb_find_file_name(path_name, target_file_name, found_file_name)
  * is returned.
  */
 EB_Error_Code
-eb_find_file_name2(path_name, sub_directory_name, target_file_name,
-    found_file_name)
-    const char *path_name;
-    const char *sub_directory_name;
-    const char *target_file_name;
-    char *found_file_name;
+eb_find_file_name2(const char *path_name, const char *sub_directory_name,
+    const char *target_file_name, char *found_file_name)
 {
     char sub_path_name[EB_MAX_PATH_LENGTH + 1];
 
@@ -473,13 +486,9 @@ eb_find_file_name2(path_name, sub_directory_name, target_file_name,
 
 
 EB_Error_Code
-eb_find_file_name3(path_name, sub_directory_name, sub2_directory_name,
-    target_file_name, found_file_name)
-    const char *path_name;
-    const char *sub_directory_name;
-    const char *sub2_directory_name;
-    const char *target_file_name;
-    char *found_file_name;
+eb_find_file_name3(const char *path_name, const char *sub_directory_name,
+    const char *sub2_directory_name, const char *target_file_name,
+    char *found_file_name)
 {
     char sub2_path_name[EB_MAX_PATH_LENGTH + 1];
 
@@ -496,23 +505,21 @@ eb_find_file_name3(path_name, sub_directory_name, sub2_directory_name,
  * and copy it into `composed_path_name'.
  */
 void
-eb_compose_path_name(path_name, file_name, composed_path_name)
-    const char *path_name;
-    const char *file_name;
-    char *composed_path_name;
+eb_compose_path_name(const char *path_name, const char *file_name,
+    char *composed_path_name)
 {
 #ifndef DOS_FILE_PATH
-    if (strcmp(path_name, "/") == 0) {
+    if (strcmp(path_name, "/") == 0)
 	sprintf(composed_path_name, "%s%s", path_name, file_name);
-    } else {
+    else
 	sprintf(composed_path_name, "%s/%s", path_name, file_name);
-    }
 #else
-    if (isalpha(*path_name) && strcmp(path_name + 1, ":\\") == 0) {
+    if (is_ebnet_url(path_name))
+	sprintf(composed_path_name, "%s/%s", path_name, file_name);
+    else if (ASCII_ISALPHA(*path_name) && strcmp(path_name + 1, ":\\") == 0)
 	sprintf(composed_path_name, "%s%s", path_name, file_name);
-    } else {
+    else
 	sprintf(composed_path_name, "%s\\%s", path_name, file_name);
-    }
 #endif
 }
 
@@ -523,12 +530,8 @@ eb_compose_path_name(path_name, file_name, composed_path_name)
  * and copy it into `composed_path_name'.
  */
 void
-eb_compose_path_name2(path_name, sub_directory_name, file_name, 
-    composed_path_name)
-    const char *path_name;
-    const char *sub_directory_name;
-    const char *file_name;
-    char *composed_path_name;
+eb_compose_path_name2(const char *path_name, const char *sub_directory_name,
+    const char *file_name, char *composed_path_name)
 {
 #ifndef DOS_FILE_PATH
     if (strcmp(path_name, "/") == 0) {
@@ -539,8 +542,12 @@ eb_compose_path_name2(path_name, sub_directory_name, file_name,
 	    path_name, sub_directory_name, file_name);
     }
 #else
-    if (isalpha(*path_name) && strcmp(path_name + 1, ":\\") == 0) {
-	sprintf(composed_path_name, "%s%s\\%s", 
+    if (is_ebnet_url(path_name)) {
+	sprintf(composed_path_name, "%s/%s/%s",
+	    path_name, sub_directory_name, file_name);
+    } else if (ASCII_ISALPHA(*path_name)
+	&& strcmp(path_name + 1, ":\\") == 0) {
+	sprintf(composed_path_name, "%s%s\\%s",
 	    path_name, sub_directory_name, file_name);
     } else {
 	sprintf(composed_path_name, "%s\\%s\\%s",
@@ -556,13 +563,9 @@ eb_compose_path_name2(path_name, sub_directory_name, file_name,
  * and copy it into `composed_path_name'.
  */
 void
-eb_compose_path_name3(path_name, sub_directory_name, sub2_directory_name,
-    file_name, composed_path_name)
-    const char *path_name;
-    const char *sub_directory_name;
-    const char *sub2_directory_name;
-    const char *file_name;
-    char *composed_path_name;
+eb_compose_path_name3(const char *path_name, const char *sub_directory_name,
+    const char *sub2_directory_name, const char *file_name,
+    char *composed_path_name)
 {
 #ifndef DOS_FILE_PATH
     if (strcmp(path_name, "/") == 0) {
@@ -573,8 +576,12 @@ eb_compose_path_name3(path_name, sub_directory_name, sub2_directory_name,
 	    path_name, sub_directory_name, sub2_directory_name, file_name);
     }
 #else
-    if (isalpha(*path_name) && strcmp(path_name + 1, ":\\") == 0) {
-	sprintf(composed_path_name, "%s%s\\%s\\%s", 
+    if (is_ebnet_url(path_name)) {
+	sprintf(composed_path_name, "%s/%s/%s/%s",
+	    path_name, sub_directory_name, sub2_directory_name, file_name);
+    } else if (ASCII_ISALPHA(*path_name)
+	&& strcmp(path_name + 1, ":\\") == 0) {
+	sprintf(composed_path_name, "%s%s\\%s\\%s",
 	    path_name, sub_directory_name, sub2_directory_name, file_name);
     } else {
 	sprintf(composed_path_name, "%s\\%s\\%s\\%s",
@@ -593,9 +600,7 @@ eb_compose_path_name3(path_name, sub_directory_name, sub2_directory_name,
  * Otherwise EB_ERR_BAD_FILE_NAME is returned.
  */
 EB_Error_Code
-eb_compose_movie_file_name(argv, composed_file_name)
-    const unsigned int *argv;
-    char *composed_file_name;
+eb_compose_movie_file_name(const unsigned int *argv, char *composed_file_name)
 {
     unsigned short jis_characters[EB_MAX_DIRECTORY_NAME_LENGTH];
     const unsigned int *arg_p;
@@ -644,9 +649,8 @@ eb_compose_movie_file_name(argv, composed_file_name)
  * EB_SUCCESS is returned upon success, EB_ERR_BAD_FILE_NAME otherwise.
  */
 EB_Error_Code
-eb_decompose_movie_file_name(argv, composed_file_name)
-    unsigned int *argv;
-    const char *composed_file_name;
+eb_decompose_movie_file_name(unsigned int *argv,
+    const char *composed_file_name)
 {
     unsigned short jis_characters[EB_MAX_DIRECTORY_NAME_LENGTH];
     unsigned int *arg_p;
@@ -678,7 +682,7 @@ eb_decompose_movie_file_name(argv, composed_file_name)
 	return EB_ERR_BAD_FILE_NAME;
 
     /*
-     * Compose file name.
+     * Decompose file name.
      */
     for (i = 0, arg_p = argv;
 	 i + 1 < EB_MAX_DIRECTORY_NAME_LENGTH; i += 2, arg_p++) {
@@ -694,15 +698,20 @@ eb_decompose_movie_file_name(argv, composed_file_name)
 
 
 void
-eb_path_name_zio_code(path_name, default_zio_code, zio_code)
-    const char *path_name;
-    Zio_Code default_zio_code;
-    Zio_Code *zio_code;
+eb_path_name_zio_code(const char *path_name, Zio_Code default_zio_code,
+    Zio_Code *zio_code)
 {
     const char *base_name;
     const char *dot;
 
+#ifndef DOS_FILE_PATH
     base_name = strrchr(path_name, '/');
+#else
+    if (is_ebnet_url(path_name))
+	base_name = strrchr(path_name, '/');
+    else
+	base_name = strrchr(path_name, '\\');
+#endif
     if (base_name != NULL)
 	base_name++;
     else
