@@ -1,16 +1,29 @@
 /*
- * Copyright (c) 1997, 98, 2000, 01  
- *    Motoyuki Kasahara
+ * Copyright (c) 1997-2004  Motoyuki Kasahara
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the project nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "build-pre.h"
@@ -22,8 +35,7 @@
  * Initialize all multi searches in the current subbook.
  */
 void
-eb_initialize_multi_searches(book)
-    EB_Book *book;
+eb_initialize_multi_searches(EB_Book *book)
 {
     EB_Subbook *subbook;
     EB_Multi_Search *multi;
@@ -53,8 +65,7 @@ eb_initialize_multi_searches(book)
  * Finalize all multi searches in the current subbook.
  */
 void
-eb_finalize_multi_searches(book)
-    EB_Book *book;
+eb_finalize_multi_searches(EB_Book *book)
 {
     EB_Subbook *subbook;
     EB_Multi_Search *multi;
@@ -83,8 +94,7 @@ eb_finalize_multi_searches(book)
  * Get information about the current subbook.
  */
 EB_Error_Code
-eb_load_multi_searches(book)
-    EB_Book *book;
+eb_load_multi_searches(EB_Book *book)
 {
     EB_Error_Code error_code;
     EB_Subbook *subbook;
@@ -105,9 +115,8 @@ eb_load_multi_searches(book)
 	/*
 	 * Read the index table page of the multi search.
 	 */
-	if (zio_lseek(&subbook->text_zio, 
-	    (off_t)(multi->search.start_page - 1) * EB_SIZE_PAGE, SEEK_SET)
-	    < 0) {
+	if (zio_lseek(&subbook->text_zio,
+	    (multi->search.start_page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
 	    error_code = EB_ERR_FAIL_SEEK_TEXT;
 	    goto failed;
 	}
@@ -225,8 +234,7 @@ static const char *default_multi_titles_latin[] = {
  * Load multi search titles.
  */
 EB_Error_Code
-eb_load_multi_titles(book)
-    EB_Book *book;
+eb_load_multi_titles(EB_Book *book)
 {
     EB_Error_Code error_code;
     EB_Subbook *subbook;
@@ -262,9 +270,8 @@ eb_load_multi_titles(book)
     /*
      * Read the page of the multi search.
      */
-    if (zio_lseek(&subbook->text_zio, 
-	(off_t)(subbook->search_title_page - 1) * EB_SIZE_PAGE, SEEK_SET)
-	< 0) {
+    if (zio_lseek(&subbook->text_zio,
+	(subbook->search_title_page - 1) * EB_SIZE_PAGE, SEEK_SET) < 0) {
 	error_code = EB_ERR_FAIL_SEEK_TEXT;
 	goto failed;
     }
@@ -334,8 +341,7 @@ succeeded:
  * or not.
  */
 int
-eb_have_multi_search(book)
-    EB_Book *book;
+eb_have_multi_search(EB_Book *book)
 {
     eb_lock(&book->lock);
     LOG(("in: eb_have_multi_search(book=%d)", (int)book->code));
@@ -368,10 +374,7 @@ eb_have_multi_search(book)
  * Return a title of the multi search `multi_id'.
  */
 EB_Error_Code
-eb_multi_title(book, multi_id, title)
-    EB_Book *book;
-    EB_Multi_Search_Code multi_id;
-    char *title;
+eb_multi_title(EB_Book *book, EB_Multi_Search_Code multi_id, char *title)
 {
     EB_Error_Code error_code;
     EB_Subbook *subbook;
@@ -428,10 +431,8 @@ eb_multi_title(book, multi_id, title)
  * Return a list of multi search ids in `book'.
  */
 EB_Error_Code
-eb_multi_search_list(book, search_list, search_count)
-    EB_Book *book;
-    EB_Multi_Search_Code *search_list;
-    int *search_count;
+eb_multi_search_list(EB_Book *book, EB_Multi_Search_Code *search_list,
+    int *search_count)
 {
     EB_Error_Code error_code;
     EB_Subbook_Code *list_p;
@@ -481,10 +482,8 @@ eb_multi_search_list(book, search_list, search_count)
  * Return the number of entries that the multi search `multi_id' in `book'.
  */
 EB_Error_Code
-eb_multi_entry_count(book, multi_id, entry_count)
-    EB_Book *book;
-    EB_Multi_Search_Code multi_id;
-    int *entry_count;
+eb_multi_entry_count(EB_Book *book, EB_Multi_Search_Code multi_id,
+    int *entry_count)
 {
     EB_Error_Code error_code;
 
@@ -540,11 +539,8 @@ eb_multi_entry_count(book, multi_id, entry_count)
  * (Legacy function)
  */
 EB_Error_Code
-eb_multi_entry_list(book, multi_id, entry_list, entry_count)
-    EB_Book *book;
-    EB_Multi_Search_Code multi_id;
-    int *entry_list;
-    int *entry_count;
+eb_multi_entry_list(EB_Book *book, EB_Multi_Search_Code multi_id,
+    int *entry_list, int *entry_count)
 {
     EB_Error_Code error_code;
     EB_Subbook_Code *list_p;
@@ -565,11 +561,8 @@ eb_multi_entry_list(book, multi_id, entry_list, entry_count)
  * Return a lable of the entry `entry_index' in the multi search `multi_id'.
  */
 EB_Error_Code
-eb_multi_entry_label(book, multi_id, entry_index, label)
-    EB_Book *book;
-    EB_Multi_Search_Code multi_id;
-    int entry_index;
-    char *label;
+eb_multi_entry_label(EB_Book *book, EB_Multi_Search_Code multi_id,
+    int entry_index, char *label)
 {
     EB_Error_Code error_code;
     EB_Subbook *subbook;
@@ -636,10 +629,8 @@ eb_multi_entry_label(book, multi_id, entry_index, label)
  * candidates or not.
  */
 int
-eb_multi_entry_have_candidates(book, multi_id, entry_index)
-    EB_Book *book;
-    EB_Multi_Search_Code multi_id;
-    int entry_index;
+eb_multi_entry_have_candidates(EB_Book *book, EB_Multi_Search_Code multi_id,
+    int entry_index)
 {
     EB_Multi_Search *multi;
 
@@ -696,11 +687,8 @@ entry_index=%d)",
  * search `multi_id'.
  */
 EB_Error_Code
-eb_multi_entry_candidates(book, multi_id, entry_index, position)
-    EB_Book *book;
-    EB_Multi_Search_Code multi_id;
-    int entry_index;
-    EB_Position *position;
+eb_multi_entry_candidates(EB_Book *book, EB_Multi_Search_Code multi_id,
+    int entry_index, EB_Position *position)
 {
     EB_Error_Code error_code;
     EB_Multi_Search *multi;
@@ -750,7 +738,7 @@ eb_multi_entry_candidates(book, multi_id, entry_index, position)
     position->page = multi->entries[entry_index].candidates_page;
     position->offset = 0;
 
-    LOG(("out: eb_multi_entry_candidates(position={%d,%d}) = %s", 
+    LOG(("out: eb_multi_entry_candidates(position={%d,%d}) = %s",
 	position->page, position->offset, eb_error_string(EB_SUCCESS)));
     eb_unlock(&book->lock);
 
@@ -760,7 +748,7 @@ eb_multi_entry_candidates(book, multi_id, entry_index, position)
      * An error occurs...
      */
   failed:
-    LOG(("out: eb_multi_entry_candidates() = %s", 
+    LOG(("out: eb_multi_entry_candidates() = %s",
 	eb_error_string(error_code)));
     eb_unlock(&book->lock);
     return error_code;
@@ -771,10 +759,8 @@ eb_multi_entry_candidates(book, multi_id, entry_index, position)
  * Multi search.
  */
 EB_Error_Code
-eb_search_multi(book, multi_id, input_words)
-    EB_Book *book;
-    EB_Multi_Search_Code multi_id;
-    const char * const input_words[];
+eb_search_multi(EB_Book *book, EB_Multi_Search_Code multi_id,
+    const char * const input_words[])
 {
     EB_Error_Code error_code;
     EB_Search_Context *context;
@@ -830,11 +816,31 @@ eb_search_multi(book, multi_id, input_words)
 	 */
 	context = book->search_contexts + word_count;
 	context->code = EB_SEARCH_MULTI;
-	context->compare_pre = eb_exact_match_canonicalized_word;
-	if (book->character_code == EB_CHARCODE_ISO8859_1)
-	    context->compare_hit = eb_exact_match_word_latin;
-	else
-	    context->compare_hit = eb_exact_match_word_jis;
+
+	/*
+	 * Choose comparison functions.
+	 */
+	if (entry->candidates_page == 0) {
+	    if (book->character_code == EB_CHARCODE_ISO8859_1) {
+		context->compare_pre    = eb_pre_match_word;
+		context->compare_single = eb_match_word;
+		context->compare_group  = eb_match_word;
+	    } else {
+		context->compare_pre    = eb_pre_match_word;
+		context->compare_single = eb_match_word;
+		context->compare_group  = eb_match_word_kana_group;
+	    }
+	} else {
+	    if (book->character_code == EB_CHARCODE_ISO8859_1) {
+		context->compare_pre    = eb_exact_pre_match_word_latin;
+		context->compare_single = eb_exact_match_word_latin;
+		context->compare_group  = eb_exact_match_word_latin;
+	    } else {
+		context->compare_pre    = eb_exact_pre_match_word_jis;
+		context->compare_single = eb_exact_match_word_jis;
+		context->compare_group  = eb_exact_match_word_kana_group;
+	    }
+	}
 	context->page = entry->start_page;
 	if (context->page == 0)
 	    continue;
