@@ -1,16 +1,29 @@
 /*
- * Copyright (c) 1997, 98, 99, 2000, 01  
- *    Motoyuki Kasahara
+ * Copyright (c) 1997-2004  Motoyuki Kasahara
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the project nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "build-pre.h"
@@ -22,10 +35,10 @@
 /*
  * Unexported functions.
  */
-static EB_Error_Code eb_narrow_character_text_jis EB_P((EB_Appendix *, int,
-    char *));
-static EB_Error_Code eb_narrow_character_text_latin EB_P((EB_Appendix *, int,
-    char *));
+static EB_Error_Code eb_narrow_character_text_jis(EB_Appendix *appendix,
+    int character_number, char *text);
+static EB_Error_Code eb_narrow_character_text_latin(EB_Appendix *appendix,
+    int character_number, char *text);
 
 /*
  * Hash macro for cache data.
@@ -38,8 +51,7 @@ static EB_Error_Code eb_narrow_character_text_latin EB_P((EB_Appendix *, int,
  * alternation or not.
  */
 int
-eb_have_narrow_alt(appendix)
-    EB_Appendix *appendix;
+eb_have_narrow_alt(EB_Appendix *appendix)
 {
     eb_lock(&appendix->lock);
     LOG(("in: eb_have_narrow_alt(appendix=%d)", (int)appendix->code));
@@ -73,9 +85,7 @@ eb_have_narrow_alt(appendix)
  * of the current subbook in `book'.
  */
 EB_Error_Code
-eb_narrow_alt_start(appendix, start)
-    EB_Appendix *appendix;
-    int *start;
+eb_narrow_alt_start(EB_Appendix *appendix, int *start)
 {
     EB_Error_Code error_code;
 
@@ -91,7 +101,7 @@ eb_narrow_alt_start(appendix, start)
     }
 
     if (appendix->subbook_current->narrow_page == 0) {
-	error_code = EB_ERR_NO_SUCH_CHAR_TEXT;
+	error_code = EB_ERR_NO_ALT;
 	goto failed;
     }
 
@@ -119,9 +129,7 @@ eb_narrow_alt_start(appendix, start)
  * of the current subbook in `book'.
  */
 EB_Error_Code
-eb_narrow_alt_end(appendix, end)
-    EB_Appendix *appendix;
-    int *end;
+eb_narrow_alt_end(EB_Appendix *appendix, int *end)
 {
     EB_Error_Code error_code;
 
@@ -137,7 +145,7 @@ eb_narrow_alt_end(appendix, end)
     }
 
     if (appendix->subbook_current->narrow_page == 0) {
-	error_code = EB_ERR_NO_SUCH_CHAR_TEXT;
+	error_code = EB_ERR_NO_ALT;
 	goto failed;
     }
 
@@ -164,10 +172,8 @@ eb_narrow_alt_end(appendix, end)
  * Get the alternation text of the character number `character_number'.
  */
 EB_Error_Code
-eb_narrow_alt_character_text(appendix, character_number, text)
-    EB_Appendix *appendix;
-    int character_number;
-    char *text;
+eb_narrow_alt_character_text(EB_Appendix *appendix, int character_number,
+    char *text)
 {
     EB_Error_Code error_code;
 
@@ -187,7 +193,7 @@ eb_narrow_alt_character_text(appendix, character_number, text)
      * The narrow font must exist in the current subbook.
      */
     if (appendix->subbook_current->narrow_page == 0) {
-	error_code = EB_ERR_NO_SUCH_CHAR_TEXT;
+	error_code = EB_ERR_NO_ALT;
 	goto failed;
     }
 
@@ -223,10 +229,8 @@ eb_narrow_alt_character_text(appendix, character_number, text)
  * Get the alternation text of the character number `character_number'.
  */
 static EB_Error_Code
-eb_narrow_character_text_jis(appendix, character_number, text)
-    EB_Appendix *appendix;
-    int character_number;
-    char *text;
+eb_narrow_character_text_jis(EB_Appendix *appendix, int character_number,
+    char *text)
 {
     EB_Error_Code error_code;
     int start;
@@ -259,7 +263,7 @@ character_number=%d)",
      * Calculate the location of alternation data.
      */
     location
-	= (off_t)(appendix->subbook_current->narrow_page - 1) * EB_SIZE_PAGE
+	= (appendix->subbook_current->narrow_page - 1) * EB_SIZE_PAGE
 	+ (((character_number >> 8) - (start >> 8)) * 0x5e
 	    + (character_number & 0xff) - (start & 0xff))
 	* (EB_MAX_ALTERNATION_TEXT_LENGTH + 1);
@@ -281,7 +285,7 @@ character_number=%d)",
 	goto failed;
     }
     cachep->character_number = -1;
-    if (zio_read(&appendix->subbook_current->zio, cachep->text, 
+    if (zio_read(&appendix->subbook_current->zio, cachep->text,
 	EB_MAX_ALTERNATION_TEXT_LENGTH + 1)
 	!= EB_MAX_ALTERNATION_TEXT_LENGTH + 1) {
 	error_code = EB_ERR_FAIL_READ_APP;
@@ -315,10 +319,8 @@ character_number=%d)",
  * Get the alternation text of the character number `character_number'.
  */
 static EB_Error_Code
-eb_narrow_character_text_latin(appendix, character_number, text)
-    EB_Appendix *appendix;
-    int character_number;
-    char *text;
+eb_narrow_character_text_latin(EB_Appendix *appendix, int character_number,
+    char *text)
 {
     EB_Error_Code error_code;
     int start;
@@ -351,7 +353,7 @@ character_number=%d)",
      * Calculate the location of alternation data.
      */
     location
-	= (off_t)(appendix->subbook_current->narrow_page - 1) * EB_SIZE_PAGE
+	= (appendix->subbook_current->narrow_page - 1) * EB_SIZE_PAGE
 	+ (((character_number >> 8) - (start >> 8)) * 0xfe
 	    + (character_number & 0xff) - (start & 0xff))
 	* (EB_MAX_ALTERNATION_TEXT_LENGTH + 1);
@@ -373,7 +375,7 @@ character_number=%d)",
 	goto failed;
     }
     cache_p->character_number = -1;
-    if (zio_read(&appendix->subbook_current->zio, cache_p->text, 
+    if (zio_read(&appendix->subbook_current->zio, cache_p->text,
 	EB_MAX_ALTERNATION_TEXT_LENGTH + 1)
 	!= EB_MAX_ALTERNATION_TEXT_LENGTH + 1) {
 	error_code = EB_ERR_FAIL_READ_APP;
@@ -407,10 +409,8 @@ character_number=%d)",
  * Return next `n'th character number from `*character_number'.
  */
 EB_Error_Code
-eb_forward_narrow_alt_character(appendix, n, character_number)
-    EB_Appendix *appendix;
-    int n;
-    int *character_number;
+eb_forward_narrow_alt_character(EB_Appendix *appendix, int n,
+    int *character_number)
 {
     EB_Error_Code error_code;
     int start;
@@ -439,7 +439,7 @@ character_number=%d)",
      * The narrow font must exist in the current subbook.
      */
     if (appendix->subbook_current->narrow_page == 0) {
-	error_code = EB_ERR_NO_SUCH_CHAR_TEXT;
+	error_code = EB_ERR_NO_ALT;
 	goto failed;
     }
 
@@ -520,10 +520,8 @@ character_number=%d)",
  * Return previous `n'th character number from `*character_number'.
  */
 EB_Error_Code
-eb_backward_narrow_alt_character(appendix, n, character_number)
-    EB_Appendix *appendix;
-    int n;
-    int *character_number;
+eb_backward_narrow_alt_character(EB_Appendix *appendix, int n,
+    int *character_number)
 {
     EB_Error_Code error_code;
     int start;
@@ -551,7 +549,7 @@ character_number=%d)",
      * The narrow font must exist in the current subbook.
      */
     if (appendix->subbook_current->narrow_page == 0) {
-	error_code = EB_ERR_NO_CUR_FONT;
+	error_code = EB_ERR_NO_ALT;
 	goto failed;
     }
 
