@@ -1,16 +1,29 @@
 /*                                                            -*- C -*-
- * Copyright (c) 1997, 98, 99, 2000, 01  
- *    Motoyuki Kasahara
+ * Copyright (c) 1997-2004  Motoyuki Kasahara
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the project nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -19,15 +32,7 @@
 
 #include <stdio.h>
 #include <sys/types.h>
-
-#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
 #include <string.h>
-#if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#include <memory.h>
-#endif /* not STDC_HEADERS and HAVE_MEMORY_H */
-#else /* not STDC_HEADERS and not HAVE_STRING_H */
-#include <strings.h>
-#endif /* not STDC_HEADERS and not HAVE_STRING_H */
 
 #ifdef ENABLE_NLS
 #ifdef HAVE_LOCALE_H
@@ -36,20 +41,10 @@
 #include <libintl.h>
 #endif
 
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr rindex
-#endif /* HAVE_STRCHR */
-
 #ifndef HAVE_STRCASECMP
-#ifdef __STDC__
 int strcasecmp(const char *, const char *);
 int strncasecmp(const char *, const char *, size_t);
-#else /* not __STDC__ */
-int strcasecmp();
-int strncasecmp();
-#endif /* not __STDC__ */
-#endif /* not HAVE_STRCASECMP */
+#endif
 
 /*
  * The maximum length of path name.
@@ -77,29 +72,37 @@ int strncasecmp();
 #define N_(string) (string)
 #endif
 #else
-#define _(string) (string)       
+#define _(string) (string)
 #define N_(string) (string)
 #endif
 
 /*
  * Character type tests and conversions.
  */
-#define isdigit(c) ('0' <= (c) && (c) <= '9')
-#define isupper(c) ('A' <= (c) && (c) <= 'Z')
-#define islower(c) ('a' <= (c) && (c) <= 'z')
-#define isalpha(c) (isupper(c) || islower(c))
-#define isalnum(c) (isupper(c) || islower(c) || isdigit(c))
-#define isxdigit(c) \
- (isdigit(c) || ('A' <= (c) && (c) <= 'F') || ('a' <= (c) && (c) <= 'f'))
-#define toupper(c) (('a' <= (c) && (c) <= 'z') ? (c) - 0x20 : (c))
-#define tolower(c) (('A' <= (c) && (c) <= 'Z') ? (c) + 0x20 : (c))
+#define ASCII_ISDIGIT(c) ('0' <= (c) && (c) <= '9')
+#define ASCII_ISUPPER(c) ('A' <= (c) && (c) <= 'Z')
+#define ASCII_ISLOWER(c) ('a' <= (c) && (c) <= 'z')
+#define ASCII_ISALPHA(c) \
+ (ASCII_ISUPPER(c) || ASCII_ISLOWER(c))
+#define ASCII_ISALNUM(c) \
+ (ASCII_ISUPPER(c) || ASCII_ISLOWER(c) || ASCII_ISDIGIT(c))
+#define ASCII_ISXDIGIT(c) \
+ (ASCII_ISDIGIT(c) || ('A' <= (c) && (c) <= 'F') || ('a' <= (c) && (c) <= 'f'))
+#define ASCII_TOUPPER(c) (('a' <= (c) && (c) <= 'z') ? (c) - 0x20 : (c))
+#define ASCII_TOLOWER(c) (('A' <= (c) && (c) <= 'Z') ? (c) + 0x20 : (c))
+
+#if defined(DOS_FILE_PATH) && defined(HAVE_MBSTRING_H)
+/* path may contain double-byte chars in SJIS. */
+#include <mbstring.h>
+#define strchr	_mbschr
+#define strrchr	_mbsrchr
+#endif
 
 /*
  * Output ``try ...'' message to standard error.
  */
 void
-output_try_help(invoked_name)
-    const char *invoked_name;
+output_try_help(const char *invoked_name)
 {
     fprintf(stderr, _("try `%s --help' for more information\n"), invoked_name);
     fflush(stderr);
@@ -110,13 +113,10 @@ output_try_help(invoked_name)
  * Output version number to stdandard out.
  */
 void
-output_version(program_name, program_version)
-    const char *program_name;
-    const char *program_version;
+output_version(const char *program_name, const char *program_version)
 {
     printf("%s (EB Library) version %s\n", program_name, program_version);
-    printf("Copyright (c) 1997, 98, 99, 2000, 01, 02\n");
-    printf("   Motoyuki Kasahara\n");
+    printf("Copyright (c) 1997-2004  Motoyuki Kasahara\n");
     fflush(stdout);
 }
 
@@ -127,11 +127,8 @@ output_version(program_name, program_version)
  * Otherwise -1 is returned.
  */
 int
-parse_subbook_name_argument(invoked_name, argument, name_list, name_count)
-    const char *invoked_name;
-    const char *argument;
-    char name_list[][EB_MAX_DIRECTORY_NAME_LENGTH + 1];
-    int *name_count;
+parse_subbook_name_argument(const char *invoked_name, const char *argument,
+    char name_list[][EB_MAX_DIRECTORY_NAME_LENGTH + 1], int *name_count)
 {
     const char *argument_p = argument;
     char name[EB_MAX_DIRECTORY_NAME_LENGTH + 1];
@@ -155,7 +152,7 @@ parse_subbook_name_argument(invoked_name, argument, name_list, name_count)
 	name_p = name;
 	while (*argument_p != ',' && *argument_p != '\0'
 	    && i < EB_MAX_DIRECTORY_NAME_LENGTH) {
-		*name_p = tolower(*argument_p);
+		*name_p = ASCII_TOLOWER(*argument_p);
 	    i++;
 	    name_p++;
 	    argument_p++;
@@ -193,10 +190,8 @@ parse_subbook_name_argument(invoked_name, argument, name_list, name_count)
  * When no sub-book is matched', EB_ERR_NO_SUCH_SUB is returned.
  */
 EB_Subbook_Code
-find_subbook(book, directory, subbook_code)
-    EB_Book *book;
-    const char *directory;
-    EB_Subbook_Code *subbook_code;
+find_subbook(EB_Book *book, const char *directory,
+    EB_Subbook_Code *subbook_code)
 {
     EB_Error_Code error_code;
     EB_Subbook_Code subbook_list[EB_MAX_SUBBOOKS];
@@ -234,8 +229,7 @@ find_subbook(book, directory, subbook_code)
  * It eliminaes `/' at the tail of `path' unless `path' is not "/".
  */
 void
-canonicalize_path(path)
-    char *path;
+canonicalize_path(char *path)
 {
     char *last_slash;
 
@@ -254,8 +248,7 @@ canonicalize_path(path)
  * It eliminaes `\' at the tail of `path' unless `path' is not "X:\".
  */
 void
-canonicalize_path(path)
-    char *path;
+canonicalize_path(char *path)
 {
     char *slash;
     char *last_backslash;
@@ -278,7 +271,7 @@ canonicalize_path(path)
     /*
      * Eliminate `\' in the tail of the path.
      */
-    if (isalpha(*path) && *(path + 1) == ':') {
+    if (ASCII_ISALPHA(*path) && *(path + 1) == ':') {
 	if (last_backslash != path + 2)
 	    *last_backslash = '\0';
     } else if (*path == '\\' && *(path + 1) == '\\') {
